#!/bin/bash 

#############################################################
# CrashPlan Linux Client Installer Script
#############################################################

# ===============================================================================
# Function definitions
# ===============================================================================

##
# Verifies a directory's existence and confirms with the user if it should be created if not found.
#
# Arguments:
# $1: string | Directory to create
#
# Returns: 0 if success, 1 if the directory failed to create for any reason.
##
prepdir() {
	if [[ ! -d "${1}" ]] ; then
		echo -n "${1} does not exist.  Create $1? (y/n) [y] "
		if [[ "${QUIET}" == false ]] ; then
			read -r YN_PD
		fi
		if [[ -z "${YN_PD}" ]] ; then
			YN_PD=y
		fi
		if [[ "${YN_PD}" == "y" ]] ; then
			if ! mkdir -p "${1}" ; then
				echo "Failed to create ${1}.  Please check your configuration."
				return 1
			fi
		else
			echo "Please check your configuration."
			return 1
		fi
	fi
	return 0
}

##
# Prints the help text
# 
# Arguments:
# $1: boolean | Long argument support. If true prints support message with long argument names.
##
printHelp() {
	if [[ -z "${1}" ]] ; then
		local SUPPORT_LONG_ARGS=true
	else 
		local SUPPORT_LONG_ARGS="${1}"
	fi

	echo "Usage: $0 [option...]" >&2
    echo
    if [[ "${SUPPORT_LONG_ARGS}" == true ]] ; then
	    echo "   -d, --deployment           Deployment options to be passed to the service on first launch."
	    echo "   -q, --quiet                Do not prompt user for input, use defaults."
	    echo "   -u, --as-current-user      Install as the current user, do not prompt for elevation."
	    echo "   -h, --help                 Show this message."
	    echo
	    echo "NOTE: Installer prefers to run as root. If running with --quiet, be sure to elevate the"
	    echo "process or run with --as-current-user as well or the installer will prompt for root"
	    echo "permissions before continuting."
	else
	    echo "   -d                         Deployment options to be passed to the service on first launch."
	    echo "   -q                         Do not prompt user for input, use defaults."
	    echo "   -u                         Install as the current user, do not prompt for elevation."
	    echo "   -h                         Show this message."
	    echo
	    echo "NOTE: Installer prefers to run as root. If running with -q, be sure to elevate the"
	    echo "process or run with -u as well or the installer will prompt for root permissions"
	    echo "before continuting."
	fi
    echo
}

##
# Parses command line arguments passed in by the invoking user
#
# Arguments:
# $1: string | All arguments captured by the shell as $@ 
##
parseArgs() {
	SHORT=rd:quh
	LONG=deployment:,quiet,as-current-user,help
	local SUPPORT_LONG_ARGS=true

	#Can we parse long arguments?
	#getopt --test > /dev/null
	
	if ! getopt --test > /dev/null ; then
		SUPPORT_LONG_ARGS=true
		PARSED="$(getopt --options ${SHORT} --longoptions ${LONG} --name "$0" -- "$@")"
	else 
		SUPPORT_LONG_ARGS=false
		PARSED="$(getopt ${SHORT} "$@")"
	fi
    
    # shellcheck disable=SC2181
	if [[ $? -ne 0 ]] ; then
	    printHelp "${SUPPORT_LONG_ARGS}"
	    exit 1
	fi

	# use eval with "$PARSED" to properly handle the quoting
	eval set -- "${PARSED}"

	while true; do
	    case "${1}" in
	        -r)
	            RECALL=true
	            shift
	            ;;
	        -q|--quiet)
	            QUIET=true
	            shift
	            ;;
	        -u|--as-current-user)
				AS_USER=true
				shift
				;;
	        -d|--deployment)
	            DEPLOYMENT="${2}"
	            shift 2
	            ;;
	        -h|--help)
				printHelp "${SUPPORT_LONG_ARGS}"
				exit 0
				;;
	        --)
	            shift
	            break
	            ;;
	        *)
	            printHelp "${SUPPORT_LONG_ARGS}"
	            exit 1
	            ;;
	    esac
	done
}

# ===============================================================================
# Begin script
# ===============================================================================

# variables defined in install.defaults
# APP_BASENAME = human-readable application name
# DIR_BASENAME = dir name
# JRE_X64_DOWNLOAD_URL = url to the 64-bit jre

SCRIPT_DIR="$(dirname "${0}")"
if [[ ! -f "${SCRIPT_DIR}/install.defaults" ]] ; then
	echo "${SCRIPT_DIR}/install.defaults MISSING!"
	exit 1
fi

# shellcheck disable=SC1090
source "${SCRIPT_DIR}/install.defaults"

REQDBINS="grep sed cpio gzip cut head tail who getopt"

QUIET=false
AS_USER=false
RECALL=false
DEPLOYMENT=""

PARENT_DIR=/usr/local
TARGETDIR=${PARENT_DIR}/${DIR_BASENAME}
BINSDIR=${PARENT_DIR}/bin
MANIFESTDIR=${PARENT_DIR}/var/${DIR_BASENAME}
INITDIR=/etc/init.d
RUNLEVEL=$(who -r | sed -e 's/^.*\(run-level [0-9]\).*$/\1/' | cut -d \  -f 2)
RUNLVLDIR=/etc/rc${RUNLEVEL}.d

SRC_USER=${SUDO_USER}
if [[ -z "${SRC_USER}" ]] ; then
	SRC_USER=${USER}
fi

USERNAME="$(id -un)"

if [[ "${USERNAME}" == "root" ]] ; then
	IS_ROOT=true
else
	IS_ROOT=false
fi

parseArgs "${@}"

# welcome- avoid printing duplicate messages if we're recalling ourself
if [[ "${RECALL}" == false ]] ; then
	echo ""
	echo "Welcome to the Code42 ${APP_BASENAME} installer."
	echo ""
	echo -n "Press enter to continue with installation. "
	if [[ "${QUIET}" == false ]] ; then
		read -r
	fi
	
	# Basic requirements - 
	echo ""
	echo "Validating environment..."
fi

# Give the user the opportunity to elevate the process if necessary
if [[ "${IS_ROOT}" == false ]] && [[ "${AS_USER}" == false ]] ; then
	echo ""
	echo "NOTE: You are apparently not installing as root. While it is recommended to"
	echo "install as root it is not required. If you continue to install as ${USERNAME}"
	echo "then ${APP_BASENAME} will only be able to back up files readable by ${USERNAME}."
	echo ""
	echo -n "Would you like to switch users and install as root? (y/n) [y] "
	if [[ "${QUIET}" == false ]] ; then
		read -r YN
	fi

	if [[ -z "${YN}" ]] ; then
		YN=y
	fi
	
	if [[ "${YN}" == "n" ]] ; then
		AS_USER=true
	fi
fi

# Basic requirements
if [[ "${IS_ROOT}" == true ]] ; then
	#If we are running as root, it doesn't matter what the state of AS_USER is.
	echo "  detected root permissions"
elif [[ "${AS_USER}" == true ]] && [[ "${IS_ROOT}" == false ]] ; then
	#If we are allowed to install as user, update vars and continue
	echo "  installing as current user"
		
	PARENT_DIR=${HOME}
	TARGETDIR=${PARENT_DIR}/${DIR_BASENAME}
	BINSDIR=
	MANIFESTDIR=${TARGETDIR}/manifest
	INITDIR=
	RUNLVLDIR=
else
	#By process of elimination, we are not root and may not run as user, elevate process.
	echo "  switching to root"
	sudo "${0}" "${@}" -r
	exit 0
fi

# ===============================================================================
# Validate the environment by verifying that all necessary binaries are present
# ===============================================================================
for BIN in $REQDBINS ; do

	if ! which "${BIN}" 2> /dev/null ; then
		echo "ERROR: $BIN not found and is required for install. Exiting"
		exit 1
	fi
done

# ===============================================================================
# On Red Hat Enterprise Linux, make sure we have libXScrnSaver installed (UI dependency)
# ===============================================================================
# This file contains bash friendly information about the operating system on RHEL & CentOS.
if [[ -e "/etc/os-release" ]] ; then
	# shellcheck disable=SC1091
	source "/etc/os-release"
	if [[ $ID =~ .*rhel.* ]] || [[ $ID_LIKE =~ .*rhel.* ]] ; then
		echo -e "\nChecking for libXScrnSaver..."
		# We're on a system that either is, or is similar to Red Hat Enterprise Linux;
		# attempt to verify that libXScrnSaver is installed using yum.
		# An exit code of '1' indicates that the package is *not* installed.
		yum list installed libXScrnSaver &>/dev/null
		if [[ $? == 1 ]] ; then
			echo "CrashPlan requires libXScrnSaver, but it was not detected on your system."
			echo -n "Would you like to install it now? (y/n) [y] "
			if [[ "${QUIET}" == false ]] ; then
				read -r YN
			fi
			if [[ -z "${YN}" ]] ; then
				YN=y
			fi

			if [[ "${YN}" == "y" ]] ; then
				echo "Installing libXScrnSaver..."
				if ! sudo yum install libXScrnSaver ; then
					# This condition will also trip if the user says "no" when yum asks the user
					# to confirm the list of packages to be installed.
					echo -e "\nInstallation was not successful.\nPlease make sure libXScrnSaver is installed."
				fi
			else
				echo -e "\nIf libXScrnSaver is not installed, CrashPlan app will not launch.\nPlease make sure this dependency is installed."
			fi
		else
			echo "Found libXScrnSaver."
		fi
	fi
fi

# ===============================================================================
# On Ubuntu, make sure libgconf-2-4 is installed (electron dependency)
# ===============================================================================

if ! ldconfig -p | grep libgconf >& /dev/null ; then
	INSTALL_LIBGCONF=false
	DISTRIB_ID=Unknown
	if [[ ${IS_ROOT} ]]; then
		if which lsb_release >& /dev/null ; then
		DISTRIB_ID=$(lsb_release -i | awk '{print $3}')
		fi 
	
		if [[ ${DISTRIB_ID} == "Ubuntu" || ${DISTRIB_ID} == "Debian" ]] ; then
			echo -en "\nlibgconf is required by CrashPlan and is not installed.  Would you like to install it? (y/n) [y] "
	
			if [[ "${QUIET}" == false ]] ; then
				read -r INSTALL_LIBGCONF_X
			fi
			if [[ -z "${INSTALL_LIBGCONF_X}" ]] ; then
				INSTALL_LIBGCONF_X=y
			fi
			if [[ "${INSTALL_LIBGCONF_X}" == "y" ]] ; then
				INSTALL_LIBGCONF=true
			fi
		fi
	fi

	if [[ ${INSTALL_LIBGCONF} == true ]] ; then
		apt-get install libgconf-2-4
	else
		echo -e "\nIf libgconf is not installed, CrashPlan app will not launch.\nPlease make sure this dependency is installed."
	fi
fi

# ===============================================================================
# Make sure to always install our JVM
# ===============================================================================
JAVACOMMON="DOWNLOAD"

# Setup ARCHIVE var to point to the cpio archive.  This will be used here to extract what we need
# to execute the Java comparison below and will be used later by the script to
# actually extract everything.
cd "${SCRIPT_DIR}" || exit 5
ARCHIVE=$(ls ./*_*.cpi)

INTERVIEW=0
while [[ ${INTERVIEW} == 0 ]] ; do

	INTERVIEWSUB=0
	while [[ ${INTERVIEWSUB} == 0 ]] ; do
		echo ""
		echo -n "What parent directory do you wish to install ${APP_BASENAME} into? [${PARENT_DIR}] "
		if [[ "${QUIET}" == false ]] ; then
			read -r PARENT_DIR_X		
		fi
		if [[ -n "${PARENT_DIR_X}" ]] ; then
			# remove trailing slash from PARENT_DIR
			PARENT_DIR_X=$(echo "${PARENT_DIR_X}" | sed '{s/\/$//g}')

			## Make sure to append "crashplan" to the directory they entered. Many enter the parent directory and assume
			## we install into a subdirectory. So if they enter /usr then we don't want to install directly into /usr
			## as that becomes a problem when they later run the uninstall script and it blows away all of /usr.
			TARGETDIR=${PARENT_DIR_X}/${DIR_BASENAME}
			MANIFESTDIR=${TARGETDIR}/manifest
		fi
		
		if prepdir "${TARGETDIR}" ; then
			INTERVIEWSUB=1
		fi
	done
		

	if [[ "${USERNAME}" == "root" ]] ; then
		INTERVIEWSUB=0
		while [[ ${INTERVIEWSUB} == 0 ]] ; do
			echo ""
			echo -n "What directory do you wish to link the ${APP_BASENAME} executable to? [${BINSDIR}] "
			if [[ "${QUIET}" == false ]] ; then
				read -r BINSDIR_X
			fi
			if [[ -n "${BINSDIR_X}" ]] ; then
				BINSDIR=${BINSDIR_X}
			fi
			if prepdir "${BINSDIR}" ; then
				INTERVIEWSUB=1
			fi
		done
	fi

	INTERVIEWSUB=0
	while [[ ${INTERVIEWSUB} == 0 ]] ; do
		echo ""
		echo -n "What directory do you wish to store incoming backup data? [${MANIFESTDIR}] "
		if [[ "${QUIET}" == false ]] ; then
			read -r MANIFESTDIR_X
		fi
		if [[ -n "${MANIFESTDIR_X}" ]] ; then
			MANIFESTDIR=${MANIFESTDIR_X}
		fi
		if prepdir "${MANIFESTDIR}" ; then
			INTERVIEWSUB=1
		fi
	done


	if [[ "${USERNAME}" == "root" ]] ; then
		INTERVIEWSUB=0
		while [[ ${INTERVIEWSUB} == 0 ]] ; do
			echo ""
			echo -n "What directory contains your SYSV init scripts? [${INITDIR}] "
			if [[ "${QUIET}" == false ]] ; then
				read -r INITDIR_X
			fi
			if [[ -n "${INITDIR_X}" ]] ; then
				INITDIR=${INITDIR_X}
			fi
			if prepdir "${INITDIR}" ; then
				INTERVIEWSUB=1
			fi
		done

		INTERVIEWSUB=0
		while [[ ${INTERVIEWSUB} == 0 ]] ; do
			echo ""
			echo -n "What directory contains your runlevel init links? [${RUNLVLDIR}] "
			if [[ "${QUIET}" == false ]] ; then
				read -r RUNLVLDIR_X
			fi
			if [[ -n "${RUNLVLDIR_X}" ]] ; then
				RUNLVLDIR=${RUNLVLDIR_X}
			fi
			if prepdir "${RUNLVLDIR}" ; then
				INTERVIEWSUB=1
			fi
		done

	fi
	echo ""
	echo "Your selections:"
	echo "${APP_BASENAME} will install to: ${TARGETDIR}"
	if [[ "${USERNAME}" == "root" ]] ; then
		echo "And put links to binaries in: ${BINSDIR}"
	fi
	echo "And store data in: ${MANIFESTDIR}"
	if [[ "${USERNAME}" == "root" ]] ; then
		echo "Your init.d dir is: ${INITDIR}"
		echo "Your current runlevel directory is: ${RUNLVLDIR}"
	fi
	echo ""
	echo -n "Is this correct? (y/n) [y] "
	if [[ "${QUIET}" == false ]] ; then
		read -r YN
	fi
	if [[ -z "${YN}" ]] ; then
		YN=y
	fi

	if [[ "${YN}" == "y" ]] ; then
		INTERVIEW=1
	fi
done

# INSTALL TIME ===============================================
echo ""

# is crashplan already there?
if [[ -f "${TARGETDIR}/install.vars" ]] ; then
	echo "CrashPlan appears to already be installed in the specified location:"
	echo "  ${TARGETDIR}"
	echo "Please uninstall and then try this install again."
	exit 1
fi

# create a file that has our install vars so we can later uninstall
{ echo ""; 
  echo "TARGETDIR=${TARGETDIR}"; 
  echo "BINSDIR=${BINSDIR}"; 
  echo "MANIFESTDIR=${MANIFESTDIR}"; 
  echo "INITDIR=${INITDIR}"; 
  echo "RUNLVLDIR=${RUNLVLDIR}"; } > "${TARGETDIR}/install.vars"
NOW=$(date +%Y%m%d)
echo "INSTALLDATE=$NOW" >> "${TARGETDIR}/install.vars"
cat "${SCRIPT_DIR}/install.defaults" >> "${TARGETDIR}/install.vars"

#download java
if [[ $JAVACOMMON == "DOWNLOAD" ]]; then
	JVMURL="${JRE_X64_DOWNLOAD_URL}"
	JVMFILE=$(basename "${JVMURL}")
	if [[ -f ${JVMFILE} ]]; then
		echo ""
		echo "Download of the JVM found. We'll try to use it, but if it's only a partial"
		echo "copy of the file then this will fail. If that happens please remove the file"
		echo "and try again."
		echo "JRE Archive: ${JVMFILE}"
		echo ""
	else
	
	    # Start by looking for wget
	    WGET_PATH=$(which wget 2> /dev/null)
	    # shellcheck disable=SC2181
	    if [[ $? == 0 ]]; then
			echo "    downloading the JRE using wget"
			$WGET_PATH "${JVMURL}"
			if [[ $? != 0 ]]; then
				echo "Unable to download JRE; please check network connection"
				exit 1
			fi
	    else

			CURL_PATH=$(which curl 2> /dev/null)
			# shellcheck disable=SC2181
			if [[ $? == 0 ]]; then
		    	echo "    downloading the JRE using curl"
				if ! $CURL_PATH -L "${JVMURL}" -o "$(basename "${JVMURL}")" ; then
					echo "Unable to download JRE; please check network connection"
					exit 1
				fi
			else
		    	echo "Could not find wget or curl.  You must install one of these utilities"
		    	echo "in order to download a JVM"
		    	exit 1
			fi
	    fi
	fi

	HERE=$(pwd)
	cd "${TARGETDIR}" || exit 5
	# Extract into ./jre
	tar -xozf "${HERE}/${JVMFILE}"
	cd "${HERE}" || exit 5
	JAVACOMMON="${TARGETDIR}/jre/bin/java"
	echo "Java Installed."
fi
echo "" >> "${TARGETDIR}/install.vars"
echo "JAVACOMMON=${JAVACOMMON}" >> "${TARGETDIR}/install.vars"

# Definition of ARCHIVE occurred above when we extracted the JAR we need to evaluate Java environment
echo "Unpacking ${HERE}/${ARCHIVE} ..." 
HERE=$(pwd)
cd "${TARGETDIR}" || exit 5
# shellcheck disable=SC2002
cat "${HERE}/${ARCHIVE}" | gzip -d -c - | cpio -i --no-preserve-owner
cd "${HERE}" || exit 5

# ATOM SHELL
mv -v "${TARGETDIR}"/*.asar "${TARGETDIR}/electron/resources"
chmod 755 "${TARGETDIR}/electron/crashplan"

# custom?
if [[ -d .Custom ]]; then
  echo "Copying .Custom to ${TARGETDIR}"
  cp -Rp .Custom "${TARGETDIR}"
fi
if [[ -d custom ]]; then
  echo "Copying custom to ${TARGETDIR}"
  cp -Rp custom "${TARGETDIR}"
fi
if [[ -d Custom ]]; then
  echo "Copying custom to ${TARGETDIR}"
  cp -Rp custom "${TARGETDIR}"
fi

#update the configs for file storage
if grep "<manifestPath>.*</manifestPath>" "${TARGETDIR}/conf/default.service.xml" > /dev/null
	then
		sed -i "s|<manifestPath>.*</manifestPath>|<manifestPath>${MANIFESTDIR}</manifestPath>|g" "${TARGETDIR}/conf/default.service.xml"
	else
		sed -i "s|<backupConfig>|<backupConfig>\n\t\t\t<manifestPath>${MANIFESTDIR}</manifestPath>|g" "${TARGETDIR}/conf/default.service.xml"
fi

# the log dir
LOGDIR=${TARGETDIR}/log
chmod 777 "${LOGDIR}"

# desktop init script
GUISCRIPT=${TARGETDIR}/bin/${APP_BASENAME}Desktop
cp "scripts/${APP_BASENAME}Desktop" "${GUISCRIPT}"
chmod 755 "${GUISCRIPT}"
#sed -imod "s|TARGETDIR=.*|TARGETDIR=${TARGETDIR}|" ${GUISCRIPT} && rm -rf ${GUISCRIPT}mod

# link to bin if appropriate
if [[ -n "${BINSDIR}" ]] ; then
	ln -s "${GUISCRIPT}" "${BINSDIR}/${APP_BASENAME}Desktop"
fi


# Install the control script for the service
INITSCRIPT=${TARGETDIR}/bin/${APP_BASENAME}Engine
cp "scripts/${APP_BASENAME}Engine" "${INITSCRIPT}"
cp scripts/run.conf "${TARGETDIR}/bin"
chmod 755 "${INITSCRIPT}"

# Install the init script and modify it by applying variables currently defined in this context
# If the user is not installing as root then we install into the bin directory only.
INIT_INSTALL_DIR=${TARGETDIR}/bin
if [[ -n "${INITDIR}" ]] ; then
   INIT_INSTALL_DIR=${INITDIR}
fi
   
# Perform substitution on the init script; we need to make the value of INITSCRIPT available
# to what lives in /etc/init.d
SEDEXPRSUB=$(echo "${INITSCRIPT}" | sed "s/\//\\\\\//g")
SEDEXPR="s/<INITSCRIPT>/${SEDEXPRSUB}/g"
sed "${SEDEXPR}" "scripts/${DIR_BASENAME}" > "${INIT_INSTALL_DIR}/${DIR_BASENAME}"
chmod 755 "${INIT_INSTALL_DIR}/${DIR_BASENAME}"

if [[ -n "${RUNLVLDIR}" ]] ; then

   # Now that we should have a working init script let's link in the runlevel scripts   
   ln -s "${INIT_INSTALL_DIR}/${DIR_BASENAME}" "${RUNLVLDIR}/S99${DIR_BASENAME}"
fi

# which icon are we using?
DESKTOP_ICON_PATH=${TARGETDIR}/bin/icon_app.png

# actual launcher will go in applications directory, but we'll create a shortcut on the desktop
LAUCHER_FILE_NAME=$(echo "${APP_BASENAME}.desktop" | awk '{print tolower($0)}')
DESKTOP_LAUNCHER="/home/${SRC_USER}/Desktop/${LAUCHER_FILE_NAME}"
# use 'su' only if we're operating as root
if [[ "${USERNAME}" == "root" ]] ; then
	APP_LAUNCHER="/usr/share/applications/${LAUCHER_FILE_NAME}"
	cp "scripts/${APP_BASENAME}.desktop" "${APP_LAUNCHER}"
	sed -imod "s|Exec=|Exec=${TARGETDIR}/|" "${APP_LAUNCHER}" && rm -rf "${APP_LAUNCHER}mod"
	sed -imod "s|Icon=.*|Icon=${DESKTOP_ICON_PATH}|" "${APP_LAUNCHER}" && rm -rf "${APP_LAUNCHER}mod"
	if [[ -d "/home/${SRC_USER}/Desktop" ]] ; then
		su "${SRC_USER}" -c "cp scripts/${APP_BASENAME}.desktop ${DESKTOP_LAUNCHER}"
		su "${SRC_USER}" -c "chmod +x ${DESKTOP_LAUNCHER}"
		su "${SRC_USER}" -c "sed -imod \"s|Exec=.*|Exec=${GUISCRIPT}|\" ${DESKTOP_LAUNCHER} && rm -rf ${DESKTOP_LAUNCHER}mod"
		su "${SRC_USER}" -c "sed -imod \"s|Icon=.*|Icon=${DESKTOP_ICON_PATH}|\" ${DESKTOP_LAUNCHER} && rm -rf ${DESKTOP_LAUNCHER}mod"
	fi
else
	APP_LAUNCHER="/home/${SRC_USER}/.local/share/applications/${LAUCHER_FILE_NAME}"
	cp "scripts/${APP_BASENAME}.desktop" "${APP_LAUNCHER}"
	sed -imod "s|Exec=|Exec=${TARGETDIR}/|" "${APP_LAUNCHER}" && rm -rf "${APP_LAUNCHER}mod"
	sed -imod "s|Icon=.*|Icon=${DESKTOP_ICON_PATH}|" "${APP_LAUNCHER}" && rm -rf "${APP_LAUNCHER}mod"
	if [[ -d "/home/${SRC_USER}/Desktop" ]] ; then
		cp "scripts/${APP_BASENAME}.desktop" "${DESKTOP_LAUNCHER}"
		chmod +x "${DESKTOP_LAUNCHER}"
		sed -imod "s|Exec=.*|Exec=${GUISCRIPT}|" "${DESKTOP_LAUNCHER}" && rm -rf "${DESKTOP_LAUNCHER}mod"
		sed -imod "s|Icon=.*|Icon=${DESKTOP_ICON_PATH}|" "${DESKTOP_LAUNCHER}" && rm -rf "${DESKTOP_LAUNCHER}mod"
	fi
fi

# Check for max_user_watches and suggest updating if necessary.  Many distros use 8192 by default
# so we use this value as a baseline.
INOTIFY_WATCHES=$(cat /proc/sys/fs/inotify/max_user_watches)
if [[ $INOTIFY_WATCHES -le 8192 ]]; then
  echo ""
  echo "Your Linux system is currently configured to watch $INOTIFY_WATCHES files in real time."
  echo "We recommend using a larger value; see the CrashPlan support site for details"
  echo ""
fi

# Start the service
${INITSCRIPT} start "${DEPLOYMENT}"

# call out the "service has been started" by creating a pause
echo ""
echo "${APP_BASENAME} has been installed and the Service has been started automatically."
echo ""
echo -n "Press Enter to complete installation. "
if [[ "${QUIET}" == false ]] ; then
	read -r
fi

echo ""
echo "Important directories:"
echo "  Installation:"
echo "    ${TARGETDIR}"
echo "  Logs:"
echo "    ${TARGETDIR}/log"
echo "  Default archive location:"
echo "    ${MANIFESTDIR}"
echo "  Readme:"
echo "    ${TARGETDIR}/doc"

# if we installed as root make sure they see 'sudo' in front of the Engine start
SUDO_PREFIX="sudo "
if [[ "${USERNAME}" != "root" ]] ; then
	SUDO_PREFIX=""
fi
echo ""
echo "Start Scripts:"
echo "  ${SUDO_PREFIX}${INITSCRIPT} start|stop"
echo "  ${GUISCRIPT}"
echo ""
if [[ "${QUIET}" == false ]] && [[ -n "${DISPLAY}" ]] ; then
	echo -n "Would you like to start ${APP_BASENAME}Desktop? (y/n) [y] "
	read -r reply
	if [[ -z "${reply}" ]] ; then
		reply=y
	fi
	case ${reply} in
		[yY] | [yY][eE][sS])
			# use 'su' only if we're operating as root
			if [[ "${USERNAME}" == "root" ]] ; then
				su "${SRC_USER}" -c "${GUISCRIPT}"
			else
				${GUISCRIPT}
			fi
			;;
	esac
fi

echo ""
echo "To start the Desktop UI:"
if [[ -n "${BINSDIR}" ]] ; then
	echo "  ${BINSDIR}/${APP_BASENAME}Desktop"
else
echo "  ${GUISCRIPT}"
fi

echo ""
echo "Installation is complete. Thank you for installing Code42 ${APP_BASENAME}."
echo ""

